@@ -1,9 +1,11 @@
 import {
     DOMWidgetModel,
     DOMWidgetView,
+    WidgetModel,
     ISerializers,
     unpack_models,
 } from '@jupyter-widgets/base';
+
 import * as utils from './utils';
 
 import { MODULE_VERSION } from './version';
@@ -14,8 +16,9 @@ export class MPLCanvasModel extends DOMWidgetModel {
     requested_size: Array<number> | null;
     resize_requested: boolean;
     ratio: number;
-    waiting: any;
+    waiting_for_image: boolean;
     image: HTMLImageElement;
+
     defaults() {
         return {
             ...super.defaults(),
@@ -32,6 +35,7 @@ export class MPLCanvasModel extends DOMWidgetModel {
             toolbar_position: 'horizontal',
             resizable: true,
             capture_scroll: false,
+            _data_url: null,
             _width: 0,
             _height: 0,
             _figure_label: 'Figure',
@@ -70,6 +74,9 @@ export class MPLCanvasModel extends DOMWidgetModel {
         this.requested_size = null;
         this.resize_requested = false;
         this.ratio = (window.devicePixelRatio || 1) / backingStore;
+
+        this.resize_canvas(this.get('_width'), this.get('_height'));
+
         this._init_image();
 
         this.on('msg:custom', this.on_comm_message.bind(this));
@@ -78,10 +85,30 @@ export class MPLCanvasModel extends DOMWidgetModel {
                 view.update_canvas();
             });
         });
+        this.on('comm_live_update', this.update_disabled.bind(this));
+
+        this.update_disabled();
 
         this.send_initialization_message();
     }
 
+    get disabled(): boolean {
+        return !this.comm_live;
+    }
+
+    update_disabled(): void {
+        this.set('resizable', !this.disabled);
+    }
+
+    sync(method: string, model: WidgetModel, options: any = {}) {
+        // Make sure we don't sync the data_url, we don't need it to be synced
+        if (options.attrs) {
+            delete options.attrs['_data_url'];
+        }
+
+        super.sync(method, model, options);
+    }
+
     send_message(type: string, message: { [index: string]: any } = {}) {
         message['type'] = type;
 
@@ -96,15 +123,15 @@ export class MPLCanvasModel extends DOMWidgetModel {
             });
         }
 
-        this.send_message('send_image_mode');
         this.send_message('refresh');
+        this.send_message('send_image_mode');
 
         this.send_message('initialized');
     }
 
     send_draw_message() {
-        if (!this.waiting) {
-            this.waiting = true;
+        if (!this.waiting_for_image) {
+            this.waiting_for_image = true;
             this.send_message('draw');
         }
     }
@@ -160,8 +187,8 @@ export class MPLCanvasModel extends DOMWidgetModel {
     }
 
     resize_canvas(width: number, height: number) {
-        this.offscreen_canvas.setAttribute('width', `${width * this.ratio}`);
-        this.offscreen_canvas.setAttribute('height', `${height * this.ratio}`);
+        this.offscreen_canvas.width = width * this.ratio;
+        this.offscreen_canvas.height = height * this.ratio;
     }
 
     handle_rubberband(msg: any) {
@@ -204,7 +231,19 @@ export class MPLCanvasModel extends DOMWidgetModel {
 
         this.image.src = image_url;
 
-        this.waiting = false;
+        this.set('_data_url', this.offscreen_canvas.toDataURL());
+
+        this.waiting_for_image = false;
+    }
+
+    handle_history_buttons(msg: any) {
+        // No-op
+    }
+
+    handle_navigate_mode(msg: any) {
+        // TODO: Remove _current_action property in the toolbar and use
+        // this message instead to know which is the current mode/which
+        // button to toggle?
     }
 
     on_comm_message(evt: any, dataviews: any) {
@@ -230,7 +269,8 @@ export class MPLCanvasModel extends DOMWidgetModel {
     }
 
     _init_image() {
-        this.image = document.createElement('img');
+        this.image = new Image();
+
         this.image.onload = () => {
             if (this.get('_image_mode') === 'full') {
                 // Full images could contain transparency (where diff images
@@ -249,6 +289,12 @@ export class MPLCanvasModel extends DOMWidgetModel {
                 view.update_canvas();
             });
         };
+
+        const dataUrl = this.get('_data_url');
+
+        if (dataUrl !== null) {
+            this.image.src = dataUrl;
+        }
     }
 
     _for_each_view(callback: any) {
@@ -275,12 +321,12 @@ export class MPLCanvasView extends DOMWidgetView {
     context: CanvasRenderingContext2D;
     top_canvas: HTMLCanvasElement;
     top_context: CanvasRenderingContext2D;
-    waiting: boolean;
     footer: HTMLDivElement;
     model: MPLCanvasModel;
     private _key: string | null;
     private _resize_event: (event: MouseEvent) => void;
     private _stop_resize_event: () => void;
+
     render() {
         this.resizing = false;
         this.resize_handle_size = 20;
@@ -303,8 +349,6 @@ export class MPLCanvasView extends DOMWidgetView {
         window.addEventListener('mousemove', this._resize_event);
         window.addEventListener('mouseup', this._stop_resize_event);
 
-        this.waiting = false;
-
         return this.create_child_view(this.model.get('toolbar')).then(
             (toolbar_view) => {
                 this.toolbar_view = toolbar_view;
@@ -509,6 +553,8 @@ export class MPLCanvasView extends DOMWidgetView {
             return;
         }
 
+        this.top_context.save();
+
         this.context.clearRect(0, 0, this.canvas.width, this.canvas.height);
         this.context.drawImage(this.model.offscreen_canvas, 0, 0);
 
@@ -524,6 +570,13 @@ export class MPLCanvasView extends DOMWidgetView {
             this.model.get('_rubberband_width') !== 0 &&
             this.model.get('_rubberband_height') !== 0
         ) {
+            this.top_context.strokeStyle = 'gray';
+            this.top_context.lineWidth = 1;
+            this.top_context.shadowColor = 'black';
+            this.top_context.shadowBlur = 2;
+            this.top_context.shadowOffsetX = 1;
+            this.top_context.shadowOffsetY = 1;
+
             this.top_context.strokeRect(
                 this.model.get('_rubberband_x'),
                 this.model.get('_rubberband_y'),
@@ -534,18 +587,19 @@ export class MPLCanvasView extends DOMWidgetView {
 
         // Draw resize handle
         if (this.model.get('resizable')) {
-            this.top_context.save();
-
             const gradient = this.top_context.createLinearGradient(
-                this.top_canvas.width - this.resize_handle_size / 3,
-                this.top_canvas.height - this.resize_handle_size / 3,
-                this.top_canvas.width - this.resize_handle_size / 4,
-                this.top_canvas.height - this.resize_handle_size / 4
+                // Start
+                this.top_canvas.width - this.resize_handle_size,
+                this.top_canvas.height - this.resize_handle_size,
+                // Stop
+                this.top_canvas.width,
+                this.top_canvas.height
             );
-            gradient.addColorStop(0, 'rgba(0, 0, 0, 0)');
-            gradient.addColorStop(1, 'rgba(0, 0, 0, 255)');
+            gradient.addColorStop(0, 'white');
+            gradient.addColorStop(1, 'black');
 
             this.top_context.fillStyle = gradient;
+            this.top_context.strokeStyle = 'gray';
 
             this.top_context.globalAlpha = 0.3;
             this.top_context.beginPath();
@@ -563,9 +617,10 @@ export class MPLCanvasView extends DOMWidgetView {
             );
             this.top_context.closePath();
             this.top_context.fill();
-
-            this.top_context.restore();
+            this.top_context.stroke();
         }
+
+        this.top_context.restore();
     }
 
     _update_cursor() {
